@@ -6,8 +6,11 @@
 #include <iomanip>
 
 #include "imms.h"
+#include "flags.h"
 #include "strmanip.h"
-#include "utils.h"
+#include "immsutil.h"
+
+#include <analyzer/distance.h>
 
 using std::string;
 using std::endl;
@@ -18,31 +21,21 @@ using std::ofstream;
 //////////////////////////////////////////////
 // Constants
 
-#define     FIRST_SKIP_RATE         -6
-#define     CONS_SKIP_RATE          -4
-#define     FIRST_NON_SKIP_RATE     5
-#define     CONS_NON_SKIP_RATE      1
-#define     JUMPED_TO_FINNISHED     7
-#define     NO_XIDLE_BONUS          1
-#define     JUMPED_FROM             -1
-#define     JUMPED_TO_SKIPPED       1
-#define     INTERACTIVE_BONUS       2
-
 #define     MAX_TIME                21*DAY
 
 #define     MAX_CORRELATION         12.0
 #define     SECONDARY               0.3
 #define     CORRELATION_IMPACT      40
 
-#define     BPM_IMPACT              25
-#define     SPECTRUM_IMPACT         10
-#define     SPECTRUM_RADIUS         3.75
-#define     BPM_RADIUS              2.5
-
 #define     LAST_EXPIRE             HOUR
 
 #define     TERM_WIDTH              80
 
+#define     SPEC_AVG                20
+#define     SPEC_IMPACT             20
+#define     BPM_AVG                 0.70
+#define     BPM_IMPACT              15
+
 //////////////////////////////////////////////
 
 // Imms
@@ -71,20 +64,26 @@ void Imms::setup(bool use_xidle)
     xidle_enabled = use_xidle;
 }
 
-void Imms::get_metacandidates()
+void Imms::get_metacandidates(int size)
 {
-    StackTimer t;
-    AutoTransaction a;
-    if (last.sid != -1)
-        CorrelationDb::get_related(metacandidates, last.sid, 20);
+    metacandidates.clear();
+
     if (handpicked.sid != -1)
         CorrelationDb::get_related(metacandidates, handpicked.sid, 30);
+    if (last.sid != -1)
+        CorrelationDb::get_related(metacandidates, last.sid, 20);
+
+    if ((int)metacandidates.size() < size)
+        PlaylistDb::get_random_sample(metacandidates,
+                size - metacandidates.size());
+
+    reverse(metacandidates.begin(), metacandidates.end());
 }
 
 void Imms::do_events()
 {
     if (!SongPicker::do_events())
-        CorrelationDb::expire_recent();
+        CorrelationDb::maybe_expire_recent();
     XIdle::query();
 }
 
@@ -96,22 +95,33 @@ void Imms::request_playlist_item(int index)
 void Imms::playlist_changed(int length)
 {
     pl_length = length;
-    local_max = pl_length * 8 * 60;
-    if (local_max > MAX_TIME)
-        local_max = MAX_TIME;
+    local_max = std::min(MAX_TIME, pl_length * 8 * 60);
 
     ImmsDb::clear_recent();
     PlaylistDb::playlist_clear();
     SongPicker::playlist_changed(length);
 } 
 
+void Imms::playlist_ready()
+{
+    PlaylistDb::playlist_ready();
+    SongPicker::playlist_ready();
+}
+
+void Imms::sync(bool incharge)
+{
+    if (!incharge)
+        PlaylistDb::clear_matches();
+    PlaylistDb::sync();
+    SongPicker::reset();
+    local_max = std::min(MAX_TIME,
+            ImmsDb::get_effective_playlist_length() * 8 * 60);
+    reset_selection();
+}
+
 void Imms::reset_selection()
 {
-    SongPicker::reset();
     server->reset_selection();
-
-    local_max = ImmsDb::get_effective_playlist_length() * 8 * 60;
-        local_max = MAX_TIME;
 }
 
 void Imms::start_song(int position, string path)
@@ -129,18 +139,22 @@ void Imms::start_song(int position, string path)
         print_song_info();
 
         if (last_jumped)
+        {
             set_lastinfo(handpicked);
-
-        StringPair acoustic = current.get_acoustic();
+            SongPicker::request_reschedule();
+        }
 
         at.commit();
-
-        if (acoustic.first == "") {
-            string epath = rex.replace(path, "'", "'\"'\"'", Regexx::global);
-            system(string("analyzer '" + epath + "' &").c_str());
-        }
     } 
     WARNIFFAILED();
+
+#ifdef ANALYZER_ENABLED
+    if (!current.get_acoustic(0, 0, 0, 0))
+    {
+        string epath = rex.replace(path, "'", "'\"'\"'", Regexx::global);
+        system(string("analyzer '" + epath + "' &").c_str());
+    }
+#endif
 }
 
 void Imms::print_song_info()
@@ -153,24 +167,24 @@ void Imms::print_song_info()
     else
         fout << path;
 
-    fout << "]\n  [Rating: " << current.rating;
+    fout << "]\n  ";
+#ifdef DEBUG
+    fout << "[" << current.get_uid() << "] ";
+#endif
+    fout << "[Rating: " << current.rating.mean;
+    fout << "(" << current.rating.dev << ")";
     fout << setiosflags(std::ios::showpos);
     if (current.relation)
         fout << current.relation << "r";
-    if (current.newness)
-        fout << current.newness << "n";
     if (current.bpmrating)
         fout << current.bpmrating << "b";
     if (current.specrating)
-        fout << current.specrating << "s";
+        fout << current.specrating << "c";
     fout << resetiosflags(std::ios::showpos);
 
     fout << "] [Last: " << strtime(current.last_played) <<
         (current.last_played == local_max ?  "!" : "");
 
-    if (current.trend_scale != 1)
-        fout << "*" << current.trend_scale;
-       
     fout << "] ";
 
     fout << (!current.identified ? "[Unknown] " : "");
@@ -182,104 +196,73 @@ void Imms::print_song_info()
 void Imms::set_lastinfo(LastInfo &last)
 {
     last.set_on = time(0);
+    last.uid = current.get_uid();
     last.sid = current.get_sid();
-    last.acoustic = current.get_acoustic();
+    last.avalid = current.get_acoustic(&last.mm, sizeof(MixtureModel),
+            last.beats, sizeof(last.beats));
 }
 
 void Imms::end_song(bool at_the_end, bool jumped, bool bad)
 {
-    int mod;
+    time_t played = at_the_end ? 10 : 5;
+    int flags = 0;
 
-    if (at_the_end)
+    if (bad)
     {
-        if (last_jumped)
-            mod = JUMPED_TO_FINNISHED;
-        else if (last_skipped)
-            mod = FIRST_NON_SKIP_RATE;
-        else
-            mod = CONS_NON_SKIP_RATE;
-
-        if (!xidle_enabled)
-            mod += NO_XIDLE_BONUS;
-        else if (XIdle::is_active())
-            mod += INTERACTIVE_BONUS;
+        played = 50;
+        flags |= Flags::bad;
     }
-    else
-    {
-        if (last_jumped && !jumped)
-            mod = JUMPED_TO_SKIPPED;
-        else if (jumped)
-            mod = JUMPED_FROM;
-        else if (last_skipped)
-            mod = CONS_SKIP_RATE;
-        else
-            mod = FIRST_SKIP_RATE;
 
-        if (mod < JUMPED_FROM)
-        {
-            if (!xidle_enabled)
-                mod -= NO_XIDLE_BONUS;
-            else if (XIdle::is_active())
-                mod -= INTERACTIVE_BONUS;
-        }
+    if (jumped)
+        flags |= Flags::jumped_from;
+    if (last_jumped)
+        flags |= Flags::jumped_to;
+
+    if ((!at_the_end && !last_skipped)
+            || (at_the_end && last_skipped))
+        flags |= Flags::first;
+
+    if (xidle_enabled)
+    {
+        flags |= Flags::idleness;
+        if (XIdle::is_active())
+            flags |= Flags::active;
     }
 
     last_skipped = !at_the_end;
 
-    if (bad)
-        mod = 0;
-
 #ifdef DEBUG
     cerr << " *** " << path_get_filename(current.get_path()) << endl;
 #endif
 
-    if (mod >= CONS_NON_SKIP_RATE)
+    if (at_the_end && (!xidle_enabled || flags & Flags::active))
         set_lastinfo(last);
 
-    if (mod > CONS_NON_SKIP_RATE + INTERACTIVE_BONUS)
+    if (at_the_end && (flags & Flags::first || flags & Flags::jumped_to))
         set_lastinfo(handpicked);
 
-    fout << (jumped ? "[Jumped] " : "");
-    fout << (!jumped && last_skipped ? "[Skipped] " : "");
-    fout << "[Delta " << setiosflags(std::ios::showpos) << mod <<
-        resetiosflags(std::ios::showpos) << "] ";
-    fout << endl;
-
     last_jumped = jumped;
 
-    int new_rating = std::min(
-            std::max(current.rating + mod, MIN_RATING), MAX_RATING);
-
-    int trend = current.get_trend();
-
-    if (abs(mod) > CONS_NON_SKIP_RATE + INTERACTIVE_BONUS)
-    {
-        if ((trend >= 0 && mod > 0) || (trend <= 0 && mod < 0))
-        {
-            trend += mod;
-            if (abs(trend) > MAX_TREND)
-                trend = trend > 0 ? MAX_TREND : - MAX_TREND;
-        }
-        else 
-            trend = mod;
-    }
-    else
-    {
-        if (abs(trend) < 3)
-            trend = 0;
-        else
-            trend += (trend > 0 ? -3 : 3);
-    }
+    Song::Rating r;
 
     AutoTransaction at;
 
-    ImmsDb::add_recent(current.get_uid(), mod);
-    current.set_trend(trend);
+    ImmsDb::add_recent(current.get_uid(), played, flags);
+    r = current.update_rating();
     current.set_last(time(0));
-    current.set_rating(new_rating);
     current.increment_playcounter();
 
     at.commit();
+
+    fout << (jumped ? "[Jumped] " : "");
+    fout << (!jumped && last_skipped ? "[Skipped] " : "");
+    fout << "[After: " << r.print() << "]";
+    fout << endl;
+
+}
+
+static inline float cap(float val, float max = 1) {
+    return std::max(std::min(val, max), -max);
 }
 
 void Imms::evaluate_transition(SongData &data, LastInfo &last, float weight)
@@ -291,29 +274,30 @@ void Imms::evaluate_transition(SongData &data, LastInfo &last, float weight)
     if (last.sid == -1)
         return;
 
-    float rel = ImmsDb::correlate(data.get_sid(), last.sid) / MAX_CORRELATION;
-    rel = std::max(std::min(rel, (float)1), (float)-1);
+    float rel = cap(ImmsDb::correlate(
+                data.get_sid(), last.sid) / MAX_CORRELATION);
     data.relation += ROUND(rel * weight * CORRELATION_IMPACT);
 
-    StringPair acoustic = data.get_acoustic();
-    if (last.acoustic.first != "" && acoustic.first != "")
+    if (!last.avalid)
+        return;
+
+    MixtureModel mm;
+    float beats[BEATSSIZE];
+    if (!data.get_acoustic(&mm, sizeof(MixtureModel), beats, sizeof(beats)))
+        return;
+    
+    float specdist = EMD::distance(last.mm, mm);
+    if (specdist >= 0)
     {
-        float d = rms_string_distance(last.acoustic.first, acoustic.first, 15);
-        float rel = (SPECTRUM_RADIUS - d) / SPECTRUM_RADIUS;
-        if (rel < 0)
-            rel *= 2;
-        data.specrating += ROUND(rel * weight * SPECTRUM_IMPACT);
+        specdist = cap((SPEC_AVG - specdist) / SPEC_AVG);
+        data.specrating += ROUND(specdist * SPEC_IMPACT * weight);
     }
 
-    if (last.acoustic.second != "" && acoustic.second != "")
+    float bpmdist = EMD::distance(last.beats, beats);
+    if (bpmdist >= 0)
     {
-        float d = rms_string_distance(
-                rescale_bpmgraph(last.acoustic.second),
-                rescale_bpmgraph(acoustic.second));
-        float rel =  (BPM_RADIUS - d) / BPM_RADIUS;
-        if (rel < 0)
-            rel *= 2;
-        data.bpmrating += ROUND(rel * weight * BPM_IMPACT);
+        bpmdist = cap((BPM_AVG - bpmdist) / BPM_AVG);
+        data.bpmrating += ROUND(bpmdist * BPM_IMPACT * weight);
     }
 }
 
