@@ -1,6 +1,7 @@
 #ifndef __SQLITE___H
 #define __SQLITE___H
 
+#include "immsconf.h"
 #include <string>
 #include <iostream>
 #include <map>
@@ -43,7 +44,7 @@ private:
 class AutoTransaction
 {
 public:
-    AutoTransaction();
+    AutoTransaction(bool exclusive = false);
     ~AutoTransaction();
     void commit();
 private:
@@ -69,7 +70,7 @@ class SQLQueryManager
 {
 public:
     SQLQueryManager() : cache(true), block(false) {}
-    sqlite3_stmt *get(const string &query);
+    sqlite3_stmt *get(const string &query, bool &cached);
     ~SQLQueryManager();
 
     static SQLQueryManager *self();
@@ -110,10 +111,12 @@ class SQLQuery
 {
 public:
     SQLQuery(const string &query);
-    ~SQLQuery() { reset(); }
+    ~SQLQuery();
 
     void reset();
     bool next();
+    bool is_null();
+    bool not_null() { return !is_null(); }
     void execute() { while (next()); }
 
     SQLQuery &operator()(int bindto) { curbind = bindto; return *this; }
@@ -124,14 +127,19 @@ public:
     SQLQuery &operator<<(const string &s);
     SQLQuery &operator<<(const SQLExec &execute);
 
+    SQLQuery &bind(const void *data, size_t n);
+
     SQLQuery &operator>>(int &i);
     SQLQuery &operator>>(long &i);
     SQLQuery &operator>>(double &i);
     SQLQuery &operator>>(float &i);
     SQLQuery &operator>>(string &s);
 
+    SQLQuery &load(void *data, size_t &n);
+
 private:
     int curbind;
+    bool cached;
 
     sqlite3_stmt *stmt;
 };
